﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.ComponentModel.DataAnnotations
Imports System.Linq
Imports System.ServiceModel.DomainServices.Server
Imports System
Imports System.Web
Imports System.Data.Objects

Public Class DataUtil
#Region "private members"

#Region "private fields"
    Private Shared m_countries As ObservableCollection(Of Country)
    Private Shared m_states As ObservableCollection(Of State)
    Private Shared m_cities As ObservableCollection(Of City)
#End Region

#Region "private methods"
    Private Shared Function CreateCountries() As ObservableCollection(Of Country)
        Dim retVal As New ObservableCollection(Of Country)

        retVal.Add(New Country With {.ID = 0, .Name = "USA"})
        retVal.Add(New Country With {.ID = 1, .Name = "CAN"})

        Dim parentCountry As Country = Nothing

        For Each s As State In States
            Dim countryID As Integer = s.CountryID
            If parentCountry Is Nothing OrElse parentCountry.ID <> countryID Then
                parentCountry = retVal.First(Function(_state) _state.ID = countryID)
            End If

            parentCountry.States.Add(s)
        Next

        Return retVal
    End Function

    Private Shared Function CreateStates() As ObservableCollection(Of State)
        Dim retVal As New ObservableCollection(Of State)

        retVal.Add(New State With {.ID = 0, .Name = "New York", .CountryID = 0})
        retVal.Add(New State With {.ID = 1, .Name = "Pennsylvania", .CountryID = 0})

        retVal.Add(New State With {.ID = 2, .Name = "Ontario", .CountryID = 1})
        retVal.Add(New State With {.ID = 3, .Name = "Quebec", .CountryID = 1})

        Dim parentState As State = Nothing

        For Each c As City In Cities
            Dim stateID As Integer = c.StateID
            If parentState Is Nothing OrElse parentState.ID <> stateID Then
                parentState = retVal.First(Function(_state) _state.ID = stateID)
            End If

            parentState.Cities.Add(c)
        Next

        Return retVal
    End Function

    Private Shared Function CreateCities() As ObservableCollection(Of City)
        Dim retVal As New ObservableCollection(Of City)

        retVal.Add(New City With {.ID = 0, .Name = "New York", .StateID = 0})
        retVal.Add(New City With {.ID = 1, .Name = "Albany", .StateID = 0})
        retVal.Add(New City With {.ID = 2, .Name = "Philadelphia", .StateID = 1})
        retVal.Add(New City With {.ID = 3, .Name = "Pittsburg", .StateID = 1})

        retVal.Add(New City With {.ID = 4, .Name = "Toronto", .StateID = 2})
        retVal.Add(New City With {.ID = 5, .Name = "Windsor", .StateID = 2})
        retVal.Add(New City With {.ID = 6, .Name = "Montreal", .StateID = 3})
        retVal.Add(New City With {.ID = 7, .Name = "Quebec City", .StateID = 3})

        Return retVal
    End Function
#End Region

#End Region

#Region "public members"

#Region "public properties"
    Public Shared ReadOnly Property Countries() As ObservableCollection(Of Country)
        Get
            If m_countries Is Nothing Then
                m_countries = CreateCountries()
            End If

            Return m_countries
        End Get
    End Property

    Public Shared ReadOnly Property States() As ObservableCollection(Of State)
        Get
            If m_states Is Nothing Then
                m_states = CreateStates()
            End If

            Return m_states
        End Get
    End Property

    Public Shared ReadOnly Property Cities() As ObservableCollection(Of City)
        Get
            If m_cities Is Nothing Then
                m_cities = CreateCities()
            End If

            Return m_cities
        End Get
    End Property
#End Region

#End Region
End Class

Public Class Country
    Implements INotifyPropertyChanged

    Private m_ID As Integer
    Private m_Name As String
    Private m_States As ObservableCollection(Of State)

    <Key()>
    Public Property ID As Integer
        Get
            Return Me.m_ID
        End Get
        Set(ByVal value As Integer)
            If Me.m_ID <> value Then
                Me.m_ID = value
                OnPropertyChanged("ID")
            End If
        End Set
    End Property

    Public Property Name As String
        Get
            Return Me.m_Name
        End Get
        Set(ByVal value As String)
            If Me.m_Name <> value Then
                Me.m_Name = value
                OnPropertyChanged("Name")
            End If
        End Set
    End Property

    <Include()>
    <Association("States", "ID", "CountryID")>
    Public ReadOnly Property States As ObservableCollection(Of State)
        Get
            If Me.m_States Is Nothing Then
                Me.m_States = New ObservableCollection(Of State)
            End If

            Return Me.m_States
        End Get
    End Property

#Region "INotifyPropertyChanged Members"
    Public Event PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs) Implements INotifyPropertyChanged.PropertyChanged

    Protected Overridable Sub OnPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
#End Region
End Class

Public Class State
    Implements INotifyPropertyChanged

    Private m_ID As Integer
    Private m_Name As String
    Private m_CountryID As Integer
    Private m_Cities As ObservableCollection(Of City)

    <Key()>
    Public Property ID As Integer
        Get
            Return Me.m_ID
        End Get
        Set(ByVal value As Integer)
            If Me.m_ID <> value Then
                Me.m_ID = value
                OnPropertyChanged("ID")
            End If
        End Set
    End Property

    Public Property Name As String
        Get
            Return Me.m_Name
        End Get
        Set(ByVal value As String)
            If Me.m_Name <> value Then
                Me.m_Name = value
                OnPropertyChanged("Name")
            End If
        End Set
    End Property

    Public Property CountryID As Integer
        Get
            Return Me.m_CountryID
        End Get
        Set(ByVal value As Integer)
            If Me.m_CountryID <> value Then
                Me.m_CountryID = value
                OnPropertyChanged("CountryID")
            End If
        End Set
    End Property

    <Include()>
    <Association("Cities", "ID", "StateID")>
    Public ReadOnly Property Cities As ObservableCollection(Of City)
        Get
            If Me.m_Cities Is Nothing Then
                Me.m_Cities = New ObservableCollection(Of City)
            End If

            Return Me.m_Cities
        End Get
    End Property

#Region "INotifyPropertyChanged Members"
    Public Event PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs) Implements INotifyPropertyChanged.PropertyChanged

    Protected Overridable Sub OnPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
#End Region
End Class

Public Class City
    Implements INotifyPropertyChanged

    Private m_ID As Integer
    Private m_Name As String
    Private m_StateID As Integer

    <Key()>
    Public Property ID As Integer
        Get
            Return Me.m_ID
        End Get
        Set(ByVal value As Integer)
            If Me.m_ID <> value Then
                Me.m_ID = value
                OnPropertyChanged("ID")
            End If
        End Set
    End Property

    Public Property Name As String
        Get
            Return Me.m_Name
        End Get
        Set(ByVal value As String)
            If Me.m_Name <> value Then
                Me.m_Name = value
                OnPropertyChanged("Name")
            End If
        End Set
    End Property

    Public Property StateID As Integer
        Get
            Return Me.m_StateID
        End Get
        Set(ByVal value As Integer)
            If Me.m_StateID <> value Then
                Me.m_StateID = value
                OnPropertyChanged("StateID")
            End If
        End Set
    End Property

#Region "INotifyPropertyChanged Members"
    Public Event PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs) Implements INotifyPropertyChanged.PropertyChanged

    Protected Overridable Sub OnPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
#End Region
End Class