﻿Imports XamGridCascadingCombos.Web
Imports System.ComponentModel
Imports System.Collections.ObjectModel
Imports System.Windows.Data
Imports Infragistics.Controls.Grids.Primitives

Partial Public Class MainPage
    Inherits UserControl

    Public Sub New()
        InitializeComponent()
    End Sub

End Class

Public Class TestDataUtil
    Private Shared m_Data As ObservableCollection(Of CityEntry)

    Public ReadOnly Property Data() As ObservableCollection(Of CityEntry)
        Get
            If m_Data Is Nothing Then
                m_Data = New ObservableCollection(Of CityEntry)
            End If

            Return m_Data
        End Get
    End Property

End Class

Public Class CityEntry
    Implements INotifyPropertyChanged

    Private m_ID As Integer
    Private m_Country As String
    Private m_State As String
    Private m_City As String

    Public Property ID As Integer
        Get
            Return Me.m_ID
        End Get
        Set(ByVal value As Integer)
            If Me.m_ID <> value Then
                Me.m_ID = value
                OnPropertyChanged("ID")
            End If
        End Set
    End Property

    Public Property Country As String
        Get
            Return Me.m_Country
        End Get
        Set(ByVal value As String)
            If Me.m_Country <> value Then
                Me.m_Country = value
                OnPropertyChanged("Country")
            End If
        End Set
    End Property

    Public Property State As String
        Get
            Return Me.m_State
        End Get
        Set(ByVal value As String)
            If Me.m_State <> value Then
                Me.m_State = value
                OnPropertyChanged("State")
            End If
        End Set
    End Property

    Public Property City As String
        Get
            Return Me.m_City
        End Get
        Set(ByVal value As String)
            If Me.m_City <> value Then
                Me.m_City = value
                OnPropertyChanged("City")
            End If
        End Set
    End Property

#Region "INotifyPropertyChanged Members"
    Public Event PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs) Implements INotifyPropertyChanged.PropertyChanged

    Protected Overridable Sub OnPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
#End Region
End Class

Public Class NameToCountryConverter
    Implements IValueConverter

    Public Function Convert(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.Convert
        Dim context As DataUtilDomainContext = TryCast(parameter, DataUtilDomainContext)

        If value IsNot Nothing AndAlso context IsNot Nothing Then
            For Each c As Country In context.Countries
                If c.Name = value.ToString() Then
                    Return c
                End If
            Next
        End If

        Return Nothing
    End Function

    Public Function ConvertBack(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.ConvertBack
        Dim retCountry As Country = TryCast(value, Country)
        If retCountry IsNot Nothing Then
            Return retCountry.Name
        End If
        Return String.Empty
    End Function
End Class

Public Class CountryNameToIDConverter
    Implements IValueConverter

    Public Function Convert(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.Convert
        Dim context As DataUtilDomainContext = TryCast(parameter, DataUtilDomainContext)

        If value <> Nothing AndAlso context IsNot Nothing Then
            For Each c As Country In context.Countries
                If c.Name = value.ToString() Then
                    Return c.ID
                End If
            Next
        End If

        Return -1
    End Function

    Public Function ConvertBack(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.ConvertBack
        Return Nothing
    End Function
End Class

Public Class NameToStateConverter
    Implements IValueConverter

    Public Function Convert(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.Convert
        Dim context As DataUtilDomainContext = TryCast(parameter, DataUtilDomainContext)

        If value IsNot Nothing AndAlso context IsNot Nothing Then
            For Each s As State In context.States
                If s.Name = value.ToString() Then
                    Return s
                End If
            Next
        End If

        Return Nothing
    End Function

    Public Function ConvertBack(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.ConvertBack
        Dim retState As State = TryCast(value, State)
        If retState IsNot Nothing Then
            Return retState.Name
        End If
        Return String.Empty
    End Function
End Class

Public Class StateNameToIDConverter
    Implements IValueConverter

    Public Function Convert(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.Convert
        Dim context As DataUtilDomainContext = TryCast(parameter, DataUtilDomainContext)

        If value <> Nothing AndAlso context IsNot Nothing Then
            For Each s As State In context.States
                If s.Name = value.ToString() Then
                    Return s.ID
                End If
            Next
        End If

        Return -1
    End Function

    Public Function ConvertBack(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.ConvertBack
        Return Nothing
    End Function
End Class

Public Class NameToCityConverter
    Implements IValueConverter

    Public Function Convert(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.Convert
        Dim context As DataUtilDomainContext = TryCast(parameter, DataUtilDomainContext)

        If value IsNot Nothing AndAlso context IsNot Nothing Then
            For Each c As City In context.Cities
                If c.Name = value.ToString() Then
                    Return c
                End If
            Next
        End If

        Return Nothing
    End Function

    Public Function ConvertBack(ByVal value As Object, ByVal targetType As System.Type, ByVal parameter As Object, ByVal culture As System.Globalization.CultureInfo) As Object Implements System.Windows.Data.IValueConverter.ConvertBack
        Dim retCity As City = TryCast(value, City)
        If retCity IsNot Nothing Then
            Return retCity.Name
        End If
        Return String.Empty
    End Function
End Class